﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Manage;
using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Share.Requests;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 系统参数相关接口
    /// </summary>
    public class SettingController : BaseController
    {
        private readonly IVoiceFileDomainService _voiceFileDomainService;
        private readonly ISystemSettingRepository _systemSettingsRepository;
        private readonly ISystemSettingGroupRepository _systemSettingGroupRepository;
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;
        

        public SettingController(IVoiceFileDomainService voiceFileDomainService, ISystemSettingRepository systemSettingsRepository, ISystemSettingGroupRepository systemSettingGroupRepository,ISystemSettingCacheManager systemSettingCacheManager)
        {
            _voiceFileDomainService = voiceFileDomainService;
            _systemSettingsRepository = systemSettingsRepository;
            _systemSettingGroupRepository = systemSettingGroupRepository;
            _systemSettingCacheManager = systemSettingCacheManager;
        }

        /// <summary>
        /// 查询语音文件
        /// </summary>
        /// <returns></returns>
        [Permission(EPermission.VoiceQueryList)]
        [HttpGet("voicequerylist")]
        public async Task<List<string>> VoiceQueryList()
        {
            return await _voiceFileDomainService.VoiceQueryListAsync(HttpContext.RequestAborted);
        }

        /// <summary>
        /// 删除语音文件
        /// </summary>
        /// <param name="voiceFileName"></param>
        /// <returns></returns>
        [Permission(EPermission.RemoveVoiceFile)]
        [HttpPost("removevoicefile")]
        public async Task RemoveVoiceFile(string voiceFileName)
        {
            await _voiceFileDomainService.RemoveVoiceFileAsync(new RemoveVoiceFileRequest(voiceFileName), HttpContext.RequestAborted);
        }

        #region 系统参数
        /// <summary>
        /// 获取系统参数列表
        /// </summary>
        /// <returns></returns>
        [Permission(EPermission.GetSysSettingsAsync)]
        [HttpGet("getsyssettings")]
        public async Task<List<SystemSettingGroup>> GetSysSettingsAsync()
        {
           //return await _systemSettingsRepository.QueryAsync(x => true);

           return await _systemSettingGroupRepository.QueryExtAsync(
               x => true, 
               x => x.Includes(d => d.SystemSettings.OrderBy(x=>x.Sort).ToList()),
               x => x.OrderBy(d=>d.Sort));

        }

        /// <summary>
        /// 更新配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [Permission(EPermission.ModifySettings)]
        [HttpPost("modifysettings")]
        public async Task ModifySettings([FromBody] ModifySettingsRequest request)
        {
            for (int i = 0; i < request.list.Count; i++)
            {
                var model = await _systemSettingsRepository.GetAsync(x => x.Id == request.list[i].id,HttpContext.RequestAborted);
                if (model != null)
                {
                    model.SettingValue = request.list[i].value;
                    await _systemSettingsRepository.UpdateAsync(model, HttpContext.RequestAborted);
                    //清除缓存
                    _systemSettingCacheManager.DelSystemSetting(model.Code);
                }
            }
        }

        #endregion

       

    }
}
