﻿using Hotline.Caching.Interfaces;
using Hotline.Caching.Services;
using Hotline.Realtimes;
using Hotline.Share.Dtos.Realtime;
using Hotline.Users;
using Microsoft.AspNetCore.SignalR;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Api.Realtimes;

public class RealtimeService : IRealtimeService, IScopeDependency
{
    private readonly IHubContext<HotlineHub> _hubContext;
    private readonly IRealtimeCacheManager _realtimeCacheManager;

    public RealtimeService(
        IHubContext<HotlineHub> hubContext,
        IRealtimeCacheManager realtimeCacheManager)
    {
        _hubContext = hubContext;
        _realtimeCacheManager = realtimeCacheManager;
    }

    /// <summary>
    /// 回铃事件通知
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public Task AlertAsync(string userId, AlertDto dto, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Alert, dto, cancellationToken);


    /// <summary>
    /// 振铃事件通知
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public Task RingAsync(string userId, RingDto dto, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Ring, dto, cancellationToken);

    /// <summary>
    /// 应答事件通知
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public Task AnsweredAsync(string userId, AnsweredDto dto, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Answered, dto, cancellationToken);


    /// <summary>
    /// 电话挂断事件通知
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public Task ByeAsync(string userId, ByeDto dto, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Bye, dto, cancellationToken);

    /// <summary>
    /// 小休审核通过后通知前端
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public Task RestApplyPassAsync(string userId, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Ring, null, cancellationToken);


    /// <summary>
    /// 通话队列通知
    /// </summary>
    /// <param name="list"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task CallQueueAsync(List<CallCacheManager.CallDtoRedis> list, CancellationToken cancellationToken)
    {
        await _hubContext.Clients.Group(RealtimeGroupNames.CallCenter)
             .SendAsync(RealtimeMethods.CallQueue, list, cancellationToken);

        //var works = _userCacheManager.GetWorks();
        //if (works != null)
        //{
        //    var sendList = works.Where(x => !string.IsNullOrEmpty(x.SignalRId)).Select(x => x.SignalRId).ToList();
        //    foreach (var work in sendList)
        //    {
        //        await _hubContext.Clients.Client(work).SendAsync(RealtimeMethods.CallQueue, list, cancellationToken);
        //    }
        //}
    }

    /// <summary>
    /// 话机空闲通知
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public Task IdleAsync(string userId, CancellationToken cancellationToken) =>
        SendCallCenterMsgAsync(userId, RealtimeMethods.Idle, new { EventName = "Idle" }, cancellationToken);

    #region private

    private async Task SendCallCenterMsgAsync(string userId, string msg, object? value, CancellationToken cancellationToken)
    {
        await CheckIfInCallCenter(userId, cancellationToken);
        var connection = await _realtimeCacheManager.GetConnectionAsync(userId, cancellationToken);
        await _hubContext.Clients.Client(connection.ConnectionId).SendAsync(msg, value, cancellationToken);
    }

    private async Task CheckIfInCallCenter(string userId, CancellationToken cancellationToken)
    {
        var isIn = await _realtimeCacheManager.IsInGroupAsync(userId, RealtimeGroupNames.CallCenter, cancellationToken);
        if (!isIn)
            throw UserFriendlyException.SameMessage("该用户不属于分组：呼叫中心");
    }

    #endregion
}